"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const chalk_1 = require("chalk");
const fs = require("fs-extra");
const getNameSpacePrefix_1 = require("../../service/getNameSpacePrefix");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages('mo-dx-plugin', 'org');
class AuraDeploy extends command_1.SfdxCommand {
    async run() {
        this.ux.startSpinner(chalk_1.default.bold.yellowBright('Saving ....'));
        const conn = this.org.getConnection();
        const namespacePrefix = await getNameSpacePrefix_1.getNameSpacePrefix(conn);
        const apiVersion = conn.getApiVersion();
        const _path = this.flags.filepath;
        const _lastindex = _path.lastIndexOf('/');
        let _fileOrDirName = _path.substring(_lastindex + 1);
        let isDirectory = false;
        let validFiles = []; // Array of all file Names to Save to Server
        let fileKey; // This is equivalent to the definitionType
        // This is when user provided the directory path
        if (_fileOrDirName === _fileOrDirName.split('.')[0]) {
            const fileNames = await fs.readdir(_path);
            isDirectory = true;
            // filter all files without xml
            validFiles = fileNames.filter(file => {
                if (file.substring(file.lastIndexOf('.') + 1) !== 'xml') {
                    return file;
                }
            });
        }
        else {
            // Below code when user provided file path and Not directory path
            validFiles.push(_fileOrDirName);
            _fileOrDirName = _fileOrDirName.split('.')[0];
        }
        try {
            const fileBodyArray = await getFileBodyMap(validFiles);
            fileKey = getFileKey(validFiles);
            let auraDefinitionBundles = [];
            auraDefinitionBundles = await getAuraDefinitionBundle(_fileOrDirName);
            if (auraDefinitionBundles.length === 0) {
                const newauraDefinitionBundle = await createAuraDefinitionBundle(_fileOrDirName);
                if (newauraDefinitionBundle.success) {
                    const auraDefinitionBundleVar = {};
                    auraDefinitionBundleVar.Id = newauraDefinitionBundle.id;
                    auraDefinitionBundles.push(auraDefinitionBundleVar);
                }
                else {
                    console.log(chalk_1.default.bold.redBright(JSON.stringify(newauraDefinitionBundle.errors)));
                }
            }
            if (auraDefinitionBundles.length > 0) {
                let auraDefinitions = await getAuraDefinitions(auraDefinitionBundles[0].Id);
                auraDefinitions = auraDefinitions.length > 0 ? auraDefinitions : [];
                try {
                    const auraDefinitionsResult = await upsertAuraDefinition(auraDefinitions, fileBodyArray, auraDefinitionBundles[0].Id);
                    this.ux.stopSpinner(chalk_1.default.bold.greenBright('AuraBundle Deployed SuccessFully..'));
                    // console.log(auraDefinitionsResult);
                }
                catch (exception) {
                    this.ux.stopSpinner(chalk_1.default.bold.redBright('Failed'));
                    console.log(chalk_1.default.bold.redBright(exception));
                }
            }
        }
        catch (exception) {
            this.ux.stopSpinner(chalk_1.default.bold.redBright('Failed'));
            console.log(chalk_1.default.bold.redBright(exception));
        }
        // function to create AuraDefinitionBundle
        async function createAuraDefinitionBundle(name) {
            const newauraDefinition = {};
            newauraDefinition.DeveloperName = name;
            newauraDefinition.MasterLabel = name;
            newauraDefinition.Description = 'A Lightning Bundle';
            newauraDefinition.ApiVersion = Number(apiVersion);
            return conn.tooling.sobject('AuraDefinitionBundle').create(newauraDefinition);
        }
        // function to get FileKey
        function getFileKey(files) {
            return files.map(file => {
                return getDefType(file.split('.')[1], file.split('.')[0]);
            });
        }
        // function to get the file body concurrently using Promise.All
        async function getFileBodyMap(files) {
            return Promise.all(files.map(async (file) => {
                const path = isDirectory ? _path + '/' + file : _path;
                return await fs.readFile(path, 'utf8');
            }));
        }
        // function to update all the AuraDefinition
        async function upsertAuraDefinition(auraDefinitions, files, bundleId) {
            const auraDefinitionsToCreate = [];
            const auraDefinitionsToUpdate = [];
            const promiseArray = [];
            fileKey.forEach(key => {
                const auraDef = auraDefinitions.find(auraDefinition => auraDefinition.DefType === key);
                if (auraDef) {
                    const definitionToUpdate = {};
                    definitionToUpdate.Id = auraDef.Id;
                    definitionToUpdate.Source = files[fileKey.indexOf(auraDef.DefType)];
                    auraDefinitionsToUpdate.push(definitionToUpdate);
                }
                else {
                    const definitionToInsert = {};
                    definitionToInsert.AuraDefinitionBundleId = bundleId;
                    definitionToInsert.DefType = key;
                    definitionToInsert.Format = getAuraFormat((validFiles[fileKey.indexOf(key)].split('.'))[1]);
                    definitionToInsert.Source = files[fileKey.indexOf(key)];
                    auraDefinitionsToCreate.push(definitionToInsert);
                }
            });
            if (auraDefinitionsToUpdate.length > 0) {
                promiseArray.push(conn.tooling.sobject('AuraDefinition').update(auraDefinitionsToUpdate));
            }
            if (auraDefinitionsToCreate.length > 0) {
                promiseArray.push(conn.tooling.sobject('AuraDefinition').create(auraDefinitionsToCreate));
            }
            return Promise.all(promiseArray);
        }
        // function to get AuraDefinitionBundleId
        async function getAuraDefinitionBundle(name) {
            return conn.tooling.sobject('AuraDefinitionBundle').find({
                DeveloperName: name,
                NamespacePrefix: namespacePrefix
            });
        }
        // function to get AuraDefinition
        async function getAuraDefinitions(bundleId) {
            return conn.tooling.sobject('AuraDefinition').find({
                AuraDefinitionBundleId: bundleId
            });
        }
        // function to get file extension
        function getDefType(extension, filename) {
            switch (extension) {
                case 'app':
                    // APPLICATION — Lightning Components app
                    return 'APPLICATION';
                case 'cmp':
                    // COMPONENT — component markup
                    return 'COMPONENT';
                case 'auradoc':
                    // DOCUMENTATION — documentation markup
                    return 'DOCUMENTATION';
                case 'css':
                    // STYLE — style (CSS) resource
                    return 'STYLE';
                case 'evt':
                    // EVENT — event definition
                    return 'EVENT';
                case 'design':
                    // DESIGN — design definition
                    return 'DESIGN';
                case 'svg':
                    // SVG — SVG graphic resource
                    return 'SVG';
                case 'js':
                    const fname = filename.toLowerCase();
                    if (fname.endsWith('controller')) {
                        return 'CONTROLLER';
                    }
                    else if (fname.endsWith('helper')) {
                        return 'HELPER';
                    }
                    else if (fname.endsWith('renderer')) {
                        // RENDERER — client-side renderer
                        return 'RENDERER';
                    }
                    break;
                case 'tokens':
                    return 'TOKENS';
                case 'intf':
                    return 'INTERFACE';
            }
        }
        // get Auraformat
        function getAuraFormat(ext) {
            // is 'js', 'css', or 'xml'
            switch (ext) {
                case 'js':
                    return 'JS';
                case 'css':
                    return 'CSS';
                default:
                    return 'XML';
            }
        }
        return '';
    }
}
AuraDeploy.description = messages.getMessage('commandDescription');
AuraDeploy.examples = [
    '$ sfdx deploy:aura -p filepath'
];
AuraDeploy.flagsConfig = {
    // flag with a value (-n, --name=VALUE)
    filepath: { type: 'string', char: 'p', description: 'file path' }
};
// Comment this out if your command does not require an org username
AuraDeploy.requiresUsername = true;
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
AuraDeploy.requiresProject = true;
exports.default = AuraDeploy;
//# sourceMappingURL=aura.js.map